var Demux = function()
{
	var demux = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};

	// Page initialization
	demux.initPage = function()
	{
		// Step 1
		Events.addEventHandler('demuxJobs', 'change', demux.demuxJobOnChange);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', demux.initializeStep2);
		Events.addEventHandler('outcomeSuccessful', 'click', demux.outcomeOnChange);
		Events.addEventHandler('outcomeFailed', 'click', demux.outcomeOnChange);
		Events.addEventHandler('deleteItemsCreatedByFailedJob', 'click', demux.outcomeOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', demux.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');	
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedDemuxJobs&pipeline=MIPS';
		Wizard.asyncJsonRequest(url, demux.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	demux.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var jobs = response.jobs;
		if (jobs.length == 0)
		{
			Wizard.setFatalError('No jobs available for confirmation.');
			return;
		}

		for (var jobNo=0; jobNo < jobs.length; jobNo++)
		{
			var job = jobs[jobNo];
			var name = job.status == 'ERROR' ? job.name + ' ( ! )' : job.name;
			if (job.partial)
			{
				name += ' [partially confirmed]';
			}
			var option = new Option(name, job.id);
			option.job = job;
			frm.demuxJobs.options[frm.demuxJobs.length] = option;
		}
		
		demux.demuxJobOnChange();

		Doc.show('step-1');
		Doc.show('gonext');

		frm.demuxJobs.focus();
	}
	
	demux.demuxJobOnChange = function()
	{
		var frm = document.forms['reggie'];	
		var job = frm.demuxJobs[frm.demuxJobs.selectedIndex].job;
		
		Doc.element('startDate').innerHTML = Reggie.reformatDate(job.started);
		Doc.element('endDate').innerHTML = Reggie.reformatDate(job.ended);
		Doc.element('jobStatus').innerHTML = '<img src="../images/'+JOB_IMAGE[job.status]+'"> '+Strings.encodeTags(job.statusMessage);

		var demuxedSequences = job.demuxedSequences;
		var fcHtml = '<table id="fc-info-table"><tr>';
		fcHtml += '<th>ID</th>';
		fcHtml += '<th>PF_READS</th>';
		fcHtml += '<th>PF_UNUSED_PCT</th>';
		fcHtml += '<th>PF_NNNN_PCT</th>';
		fcHtml += '<th>Demultiplex metrics</th>';
		fcHtml += '<th>Warnings</th></tr>';
		var fcNames = '';
		var poolHtml = '';
		var libPlateHtml = '';
		var warningsHtml = '';
		var skippedTilesHtml = '';
		for (var dxNo = 0; dxNo < demuxedSequences.length; dxNo++)
		{
			var dx = demuxedSequences[dxNo];
			var fc = dx.seqRun.flowCell;
			
			var pfReads = dx.PF_READS == null ? '-' : Reggie.formatNumber(dx.PF_READS/1000000, 'M', 2);
			var pfUnused =  dx.PF_UNUSED_PCT == null ? '-' : Reggie.formatNumber(dx.PF_UNUSED_PCT, '%', 2);
			var pfNNNN = dx.PF_NNNN_PCT == null ? '-' : Reggie.formatNumber(dx.PF_NNNN_PCT, '%', 2);

			if (fcNames.length > 0) fcNames += ', ';
			fcNames += Strings.encodeTags(fc.FlowCellID);
			
			fcHtml += '<tr><td>'+Strings.encodeTags(fc.FlowCellID)+'</td>';
			fcHtml += '<td>' + pfReads + '</td>';
			fcHtml += '<td>' + pfUnused + ' </td>';
			fcHtml += '<td>' + pfNNNN + '</td>';
			
			fcHtml += '<td>';
			if  (dx.DemultiplexMetrics)
			{
				fcHtml += '<span class="link metrics-link" data-file-id="'+dx.DemultiplexMetrics.id+'" title="View metrics data from the demux">'+dx.DemultiplexMetrics.name+'</span>';
			}
			fcHtml += '</td>';
			
			fcHtml += '<td>'+dx.DEMUX_WARNING.length+'</td>';
			fcHtml += '</tr>';
			
			var pools = fc.pools;
			var poolNames = [];
			var libPlateNames = [];
			for (var poolNo = 0; poolNo < pools.length; poolNo++)
			{
				var pool = pools[poolNo];
				poolNames[poolNames.length] = pool.name;
				if (pool.libPlates)
				{
					for (var plateNo = 0; plateNo < pool.libPlates.length; plateNo++)
					{
						var plateName = pool.libPlates[plateNo].name;
						if (libPlateNames.indexOf(plateName) == -1)
						{
							libPlateNames[libPlateNames.length] = plateName;
						}
					}
				}
			}
			
			if (dx.DEMUX_WARNING.length > 0)
			{
				for (var warnNo = 0; warnNo < dx.DEMUX_WARNING.length; warnNo++)
				{
					warningsHtml += '<li class="warning">'+Strings.encodeTags(fc.FlowCellID)+': '+Strings.encodeTags(dx.DEMUX_WARNING[warnNo]);
				}
			}
			
			if (dx.SKIPPED_TILES.length > 0)
			{
				dx.SKIPPED_TILES.sort();
				var lastLaneNo = 0;
				for (var tileNo = 0; tileNo < dx.SKIPPED_TILES.length; tileNo++)
				{
					var tmp = dx.SKIPPED_TILES[tileNo];
					var i = tmp.indexOf(':');
					var laneNo = tmp.substring(0, i);
					var tile = tmp.substring(i+1);
					if (lastLaneNo != laneNo)
					{
						skippedTilesHtml += '<li class="warning">' + Strings.encodeTags(fc.FlowCellID) + ' - lane ' +laneNo + ': ';
						lastLaneNo = laneNo;
					}
					else
					{
						skippedTilesHtml += ', ';
					}
					skippedTilesHtml += Strings.encodeTags(tile);
				}
			}
		}
		fcHtml += '</table>';

		Doc.element('flowCellsInDemux').innerHTML = job.status == 'ERROR' ? fcNames : fcHtml;
		Doc.element('pools').innerHTML = Strings.encodeTags(poolNames.join(', '));
		Doc.element('libPlates').innerHTML = Strings.encodeTags(libPlateNames.join(', ')) || 'n/a';
		Doc.element('comments').innerHTML = Strings.encodeTags(job.comments);
		Doc.element('warnings').innerHTML = warningsHtml;
		Doc.element('skippedTiles').innerHTML = skippedTilesHtml;
		Doc.showHide('skippedTilesRow', skippedTilesHtml != '');
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('metrics-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Files.viewFileOnClick);
		}

	}

	
	demux.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		var job = frm.demuxJobs[frm.demuxJobs.selectedIndex].job;
		
		if (job.partial)
		{
			Doc.element('outcomeFailed').disabled = true;
			Wizard.setInputStatus('outcome', 'warning', 'Some libraries have already been confirmed.');
		}

		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=GetMergedSequences&job='+job.id;
		
		Wizard.showLoadingAnimation('Loading merged sequences...');
		Wizard.asyncJsonRequest(url, demux.mergedSequencesLoaded);
	}
	
	demux.mergedSequencesLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var job = frm.demuxJobs[frm.demuxJobs.selectedIndex].job;
		var isError = job.status == 'ERROR';
		
		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('gocancel');
		
		var mergedSeq = response.mergedSequences;
		mergedSeq.sort(demux.sortByPool);
		job.mergedSequences = mergedSeq;
			
		var html = '<table id="mergedSequencesTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="3">Reads (Thousands)</th>';
		html += '<th class="dottedleft" colspan="2">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>Library</th>';
		html += '<th class="dottedleft">Pools</th>';
		html += '<th class="dottedleft">READS</th>';
		html += '<th>PF_READS</th>';
		html += '<th class="warning-col"></th>';
		html += '<th class="dottedleft">Flag DNA</th>';
		html += '<th>????</th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var yellowImg = '<img src="../images/yellow-label.png">';
		var lastPoolName = mergedSeq[0].pools[0].name;
		for (var mergedNo = 0; mergedNo < mergedSeq.length; mergedNo++)
		{
			var merged = mergedSeq[mergedNo];
			var lib = merged.lib;
			var pools = merged.pools;
			var warning = null;
			
			merged.stratagene = Reggie.isStratagene(merged.name);
			merged.external = Reggie.isExternal(merged.name);
			var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
			var img = isYellow ? yellowImg : '';

			var reads = merged.READS == null ? '-' : Reggie.formatNumber(merged.READS/1000, null, 2);
			var pfReads = merged.PF_READS == null ? '-' : Reggie.formatNumber(merged.PF_READS/1000, null, 2);

			var flagChecked = false;
			var alignChecked = !merged.stratagene;
			merged.alreadyConfirmed = merged.AnalysisResult != null;
			
			html += '<tr class="highlight ' + (isYellow ? 'yellow-specimen' : '') + '"';
			if (pools[0].name != lastPoolName)
			{
				lastPoolName = pools[0].name;
				html += 'style="border-top: 1px solid #000000;"';
			}
			html += '><td class="prompt if-yellow">'+img+Strings.encodeTags(merged.name)+'</td>';
			html += '<td class="dottedleft">';
			for (var poolNo = 0; poolNo < pools.length; poolNo++)
			{
				if (poolNo > 0) html += ', ';
				html += Strings.encodeTags(pools[poolNo].name);
			}
			html += '</td>';
			html += '<td class="dottedleft">'+reads+'</td>';
			html += '<td>'+pfReads+'</td>';
			html += '<td class="warning-col">';
			html += '</td>';

			if (merged.alreadyConfirmed)
			{
				html += '<td class="dottedleft" colspan="2"></td>';
				html += '<td class="dottedleft italic comment" style="padding-left: 2px;">Already confirmed</td>';
			}
			else
			{
				// Do not allow flagging external or stratagene
				if (merged.stratagene || merged.external)
				{
					html += '<td class="dottedleft">-</td>';
				}
				else
				{
					html += '<td class="dottedleft"><input type="checkbox" name="flag.'+merged.id+'"'+(flagChecked ? ' checked':'')+'></td>';
				}
					
				html += '<td><input type="checkbox" name="nextStepTODO.'+merged.id+'" '+(alignChecked ? ' checked':'')+'></td>';
				html += '<td class="dottedleft comment"><input type="text" name="comment.'+merged.id+'"></td>';
			}
			html += '</tr>';
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('mergedSequences').innerHTML = html;
			
		if (isError)
		{
			Doc.element('outcomeSuccessful').disabled = true;
			Doc.element('outcomeFailed').checked = true;
			frm.deleteItemsCreatedByFailedJob.checked = true;
			frm.demuxAgain.checked = true;
			demux.outcomeOnChange();
		}
	}
	
	/**
		Sort by the name of the first pool.
	*/
	demux.sortByPool = function(a, b)
	{
		var p1 = a.pools[0].name;
		var p2 = b.pools[0].name;
		if (p1 < p2) return -1;
		if (p1 > p2) return 1;
		if (a.name < b.name) return -1;
		if (a.name > b.name) return 1;
		return a.id - b.id;
	}

	demux.outcomeOnChange = function()
	{
		var frm = document.forms['reggie'];	
		var failed = Doc.element('outcomeFailed').checked;
		var deleteItems = frm.deleteItemsCreatedByFailedJob.checked;
		frm.demuxAgain.disabled = !failed;
		frm.deleteItemsCreatedByFailedJob.disabled = !failed;
		if (frm.demuxAgain.disabled) frm.demuxAgain.checked = false;
		if (frm.deleteItemsCreatedByFailedJob.disabled) frm.deleteItemsCreatedByFailedJob.checked = false;
		
		var job = frm.demuxJobs[frm.demuxJobs.selectedIndex].job;
		var mergedSequences = job.mergedSequences;
		
		for (var mergeNo = 0; mergeNo < mergedSequences.length; mergeNo++)
		{
			var merge = mergedSequences[mergeNo];
			if (frm['flag.'+merge.id])
			{
				frm['flag.'+merge.id].disabled = failed;
				demux.resetCheckbox(frm['flag.'+merge.id]);
			}
			if (frm['nextStepTODO.'+merge.id])
			{
				frm['nextStepTODO.'+merge.id].disabled = failed;
				demux.resetCheckbox(frm['nextStepTODO.'+merge.id]);
			}
			if (frm['comment.'+merge.id])
			{
				frm['comment.'+merge.id].disabled = deleteItems;
			}
		}
	}

	/**
		A disabled checkbox is forced to be unchecked, the old value is stored in
		'oldChecked'. An enabled checkbox with 'oldChecked' set is checked.
	 */
	demux.resetCheckbox = function(checkbox)
	{
		if (checkbox.disabled)
		{
			checkbox.oldChecked = checkbox.checked;
			checkbox.checked = false;
		}
		else if (checkbox.oldChecked != null)
		{
			checkbox.checked = checkbox.oldChecked;
			checkbox.oldChecked = null;
		}
	}
	
	demux.submit = function()
	{

		var frm = document.forms['reggie'];	
		var failed = Doc.element('outcomeFailed').checked;
		var submitInfo = {};
		submitInfo.failed = failed;
		submitInfo.deleteItemsCreatedByFailedJob = frm.deleteItemsCreatedByFailedJob.checked;
		submitInfo.demuxAgain = frm.demuxAgain.checked;
		
		var job = frm.demuxJobs[frm.demuxJobs.selectedIndex].job;
		var demuxedSequences = job.demuxedSequences;
		var mergedSequences = job.mergedSequences;
			
		var dx = [];
		submitInfo.demuxedSequences = dx;
		for (var demuxNo = 0; demuxNo < demuxedSequences.length; demuxNo++)
		{
			var tmp = {};
			tmp.id = demuxedSequences[demuxNo].id;
			dx[dx.length] = tmp;
		}
		
		var merged = [];
		submitInfo.mergedSequences = merged;
		for (var mergeNo = 0; mergeNo < mergedSequences.length; mergeNo++)
		{
			var merge = mergedSequences[mergeNo];
			if (!merge.alreadyConfirmed)
			{
				var tmp = {};
				tmp.id = merge.id;
				tmp.flag = frm['flag.'+merge.id] && frm['flag.'+merge.id].checked;
				tmp.nextStepTODO = frm['nextStepTODO.'+merge.id].checked;
				tmp.comment = frm['comment.'+merge.id].value;
				merged[merged.length] = tmp;
			}
		}
		
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterDemuxAndMerge&pipeline=MIPS';
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, demux.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	demux.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return demux;
}();

Doc.onLoad(Demux.initPage);

